@@ -4,6 +4,8 @@
 
 // #[cfg(any(feature = "server", feature = "client"))]
 mod util;
+#[cfg(all(feature = "client", feature = "runtime"))]
+use darling::FromMeta;
 // #[cfg(any(feature = "server", feature = "client"))]
 use util::item_impl::*;
 // #[cfg(any(feature = "server", feature = "client"))]
@@ -15,15 +17,9 @@ pub(crate) const ATTR_EXPORT_METHOD: &str = "export_method";
 pub(crate) const HANDLER_SUFFIX: &str = "handler";
 #[cfg(feature = "server")]
 pub(crate) const EXPORTED_TRAIT_SUFFIX: &str = "Handler";
-#[cfg(all(
-    feature = "client",
-    feature = "runtime",
-))]
+#[cfg(all(feature = "client", feature = "runtime",))]
 pub(crate) const CLIENT_SUFFIX: &str = "Client";
-#[cfg(all(
-    feature = "client",
-    feature = "runtime",
-))]
+#[cfg(all(feature = "client", feature = "runtime",))]
 pub(crate) const CLIENT_STUB_SUFFIX: &str = "ClientStub";
 
 /// A macro that impls serde::Deserializer by simply calling the
@@ -232,7 +228,6 @@ pub fn impl_inner_deserializer(_: proc_macro::TokenStream) -> proc_macro::TokenS
     output.into()
 }
 
-
 // =============================================================================
 // #[export_impl]
 // =============================================================================
@@ -244,17 +239,17 @@ pub fn impl_inner_deserializer(_: proc_macro::TokenStream) -> proc_macro::TokenS
 /// Under the hood, this macro generates method handlers. This macro implements the
 /// `toy_rpc::util::RegisterService` trait, which returns the handler hashmap
 /// when a service is registered on the server.
-/// 
+///
 /// ### Note
-/// 
+///
 /// - The default service name generated will be the same as the name of the struct.
 ///
 /// ### Example - Export impl block
-/// 
+///
 /// ```rust
 /// struct Abacus { }
-/// 
-/// #[export_impl]
+///
+/// #[export_impl] // This will give a default service name of "Abacus"
 /// impl Abacus {
 ///     #[export_method]
 ///     async fn subtract(&self, args(i32, i32)) -> Result<i32, String> {
@@ -263,20 +258,17 @@ pub fn impl_inner_deserializer(_: proc_macro::TokenStream) -> proc_macro::TokenS
 /// }
 /// ```
 #[proc_macro_attribute]
-pub fn export_impl(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream) -> proc_macro::TokenStream {
+pub fn export_impl(
+    _attr: proc_macro::TokenStream,
+    item: proc_macro::TokenStream,
+) -> proc_macro::TokenStream {
     // parse item
     let input = syn::parse_macro_input!(item as syn::ItemImpl);
     #[cfg(feature = "server")]
     let (handler_impl, names, handler_idents) = transform_impl(input.clone());
 
     // extract Self type and use it for construct Ident for handler HashMap
-    #[cfg(any(
-        feature = "server", 
-        all(
-            feature = "client",
-            feature = "runtime"
-        )
-    ))]
+    #[cfg(any(feature = "server", all(feature = "client", feature = "runtime")))]
     let ident = {
         let self_ty = &input.self_ty;
         match util::parse_impl_self_ty(self_ty) {
@@ -299,11 +291,7 @@ pub fn export_impl(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream
     #[cfg(all(feature = "client", feature = "runtime"))]
     let client_impl = remove_export_attr_from_impl(client_impl);
 
-    #[cfg(all(
-        feature = "server",
-        feature = "client",
-        feature = "runtime"
-    ))]
+    #[cfg(all(feature = "server", feature = "client", feature = "runtime"))]
     let output = quote::quote! {
         #input
         #handler_impl
@@ -313,11 +301,7 @@ pub fn export_impl(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream
         #stub_trait
         #stub_impl
     };
-    #[cfg(all(
-        not(feature = "server"),
-        feature = "client",
-        feature = "runtime"
-    ))]
+    #[cfg(all(not(feature = "server"), feature = "client", feature = "runtime"))]
     let output = quote::quote! {
         #input
         #client_ty
@@ -327,10 +311,7 @@ pub fn export_impl(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream
     };
     #[cfg(all(
         feature = "server",
-        any(
-            not(feature = "client"),
-            not(feature = "runtime")
-        )
+        any(not(feature = "client"), not(feature = "runtime"))
     ))]
     let output = quote::quote! {
         #input
@@ -339,10 +320,7 @@ pub fn export_impl(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream
     };
     #[cfg(all(
         not(feature = "server"),
-        any(
-            not(feature = "client"),
-            not(feature = "runtime")
-        )
+        any(not(feature = "client"), not(feature = "runtime"))
     ))]
     let output = quote::quote! {
         #input
@@ -354,87 +332,128 @@ pub fn export_impl(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream
 // #[export_trait]
 // =============================================================================
 
-/// "Exports" methods defined in the trait with the `#[export_method]` attribute. 
-/// Methods not marked with `#[export_method]` will not be affected. 
-/// 
-/// This macro should be used together with `#[export_trait_impl]` to allow conveniently 
-/// register the struct that implements the service trait as a service. 
-/// 
+#[derive(Debug, darling::FromMeta)]
+struct MacroArgs {
+    #[darling(default)]
+    impl_for_client: bool,
+}
+
+/// "Exports" methods defined in the trait with the `#[export_method]` attribute.
+/// Methods not marked with `#[export_method]` will not be affected.
+///
+/// This macro should be used together with `#[export_trait_impl]` to allow conveniently
+/// register the struct that implements the service trait as a service.
+///
 /// ## Note
-/// 
-/// - The default service name generated will be the same as the name of the trait. 
-/// 
-/// - This macro should be placed on the trait definition. 
-/// 
+///
+/// - The default service name generated will be the same as the name of the trait.
+///
+/// - This macro should be placed on the trait definition.
+///
 /// ## Example
-/// 
-/// ```rust 
+///
+/// ```rust
 /// #[async_trait]
-/// #[export_trait]
+/// #[export_trait] // This will give a default service name of "Arith"
 /// pub trait Arith {
-///     // Mark method(s) to be "exported" with `#[export_method]` 
+///     // Mark method(s) to be "exported" with `#[export_method]`
 ///     // in the definition.
 ///     #[export_method]
 ///     async fn add(&self, args(i32, i32)) -> Result<i32, String>;
 /// }
 /// ```
 #[proc_macro_attribute]
-pub fn export_trait(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream) -> proc_macro::TokenStream {
+pub fn export_trait(
+    _attr: proc_macro::TokenStream,
+    item: proc_macro::TokenStream,
+) -> proc_macro::TokenStream {
+    #[cfg(all(feature = "client", feature = "runtime"))]
+    let args = {
+        let attr_args = syn::parse_macro_input!(_attr as syn::AttributeArgs);
+        match MacroArgs::from_list(&attr_args) {
+            Ok(v) => v,
+            Err(err) => {
+                return proc_macro::TokenStream::from(err.write_errors());
+            }
+        }
+    };
+
     let input = syn::parse_macro_input!(item as syn::ItemTrait);
     #[cfg(feature = "server")]
-    let (transformed_trait, 
-        transformed_trait_impl,
-        names, 
-        handler_idents
-    ) = transform_trait(input.clone());
+    let (transformed_trait, transformed_trait_impl, names, handler_idents) =
+        transform_trait(input.clone());
     #[cfg(feature = "server")]
     let local_registry = impl_local_registry_for_trait(
-        &input.ident, &transformed_trait.ident, names, handler_idents);
+        &input.ident,
+        &transformed_trait.ident,
+        names,
+        handler_idents,
+    );
 
     #[cfg(all(feature = "client", feature = "runtime"))]
     let (client_ty, client_impl) = generate_service_client_for_trait(&input.ident, &input);
     #[cfg(all(feature = "client", feature = "runtime"))]
     let (stub_trait, stub_impl) = generate_client_stub_for_trait(&input.ident);
 
+    #[cfg(all(feature = "client", feature = "runtime"))]
+    let trait_impl = {
+        let trait_impl = generate_trait_impl_for_client(&input);
+        remove_export_attr_from_impl(trait_impl)
+    };
+
     let input = remove_export_attr_from_trait(input);
     #[cfg(feature = "server")]
     let transformed_trait = remove_export_attr_from_trait(transformed_trait);
     #[cfg(feature = "server")]
     let transformed_trait_impl = remove_export_attr_from_impl(transformed_trait_impl);
 
-    #[cfg(all(
-        feature = "server",
-        feature = "client",
-        feature = "runtime"
-    ))]
-    let output = quote::quote! {
-        #input
-        #transformed_trait
-        #transformed_trait_impl
-        #local_registry
-        #client_ty
-        #client_impl
-        #stub_trait
-        #stub_impl
+    #[cfg(all(feature = "server", feature = "client", feature = "runtime"))]
+    let output = if args.impl_for_client {
+        quote::quote! {
+            #input
+            #transformed_trait
+            #transformed_trait_impl
+            #local_registry
+            #client_ty
+            #client_impl
+            #stub_trait
+            #stub_impl
+            #trait_impl
+        }
+    } else {
+        quote::quote! {
+            #input
+            #transformed_trait
+            #transformed_trait_impl
+            #local_registry
+            #client_ty
+            #client_impl
+            #stub_trait
+            #stub_impl
+        }
     };
-    #[cfg(all(
-        not(feature = "server"),
-        feature = "client",
-        feature = "runtime"
-    ))]
-    let output = quote::quote! {
-        #input
-        #client_ty
-        #client_impl
-        #stub_trait
-        #stub_impl
+    #[cfg(all(not(feature = "server"), feature = "client", feature = "runtime"))]
+    let output = if args.impl_for_client {
+        quote::quote! {
+            #input
+            #client_ty
+            #client_impl
+            #stub_trait
+            #stub_impl
+            #trait_impl
+        }
+    } else {
+        quote::quote! {
+            #input
+            #client_ty
+            #client_impl
+            #stub_trait
+            #stub_impl
+        }
     };
     #[cfg(all(
         feature = "server",
-        any(
-            not(feature = "client"),
-            not(feature = "runtime")
-        )
+        any(not(feature = "client"), not(feature = "runtime"))
     ))]
     let output = quote::quote! {
         #input
@@ -444,10 +463,7 @@ pub fn export_trait(_attr: proc_macro::TokenStream, item: proc_macro::TokenStrea
     };
     #[cfg(all(
         not(feature = "server"),
-        any(
-            not(feature = "client"),
-            not(feature = "runtime")
-        )
+        any(not(feature = "client"), not(feature = "runtime"))
     ))]
     let output = quote::quote! {
         #input
@@ -457,20 +473,20 @@ pub fn export_trait(_attr: proc_macro::TokenStream, item: proc_macro::TokenStrea
 
 /// This macro implements the `toy_rpc::util::RegisterService` trait to allow
 /// convenient registration of the service. This should be used along with the
-/// macro `#[export_trait]`. 
-/// 
+/// macro `#[export_trait]`.
+///
 /// ## Note
-/// 
-/// - This macro should be placed on the impl block of the defined RPC service 
-/// trait 
-/// 
-/// ## Example 
-/// 
-/// ```rust 
+///
+/// - This macro should be placed on the impl block of the defined RPC service
+/// trait
+///
+/// ## Example
+///
+/// ```rust
 /// struct Abacus { }
-/// 
+///
 /// #[async_trait]
-/// #[export_trait_impl]
+/// #[export_trait_impl] // The default name will follow the name of the trait (ie. "Arith")
 /// impl Arith  for Abacus {
 ///     // Notice that you do NOT mark the method with `#[export_method]`
 ///     // again
@@ -480,7 +496,10 @@ pub fn export_trait(_attr: proc_macro::TokenStream, item: proc_macro::TokenStrea
 /// }
 /// ```
 #[proc_macro_attribute]
-pub fn export_trait_impl(_attr: proc_macro::TokenStream, item: proc_macro::TokenStream) -> proc_macro::TokenStream {
+pub fn export_trait_impl(
+    _attr: proc_macro::TokenStream,
+    item: proc_macro::TokenStream,
+) -> proc_macro::TokenStream {
     let input = syn::parse_macro_input!(item as syn::ItemImpl);
     #[cfg(feature = "server")]
     let trait_ident = get_trait_ident_from_item_impl(&input).unwrap();
@@ -497,7 +516,7 @@ pub fn export_trait_impl(_attr: proc_macro::TokenStream, item: proc_macro::Token
 
     #[cfg(feature = "server")]
     let register_impl = impl_register_service_for_trait_impl(&trait_ident, type_ident);
-    
+
     let input = remove_export_attr_from_impl(input);
 
     #[cfg(feature = "server")]
@@ -510,4 +529,57 @@ pub fn export_trait_impl(_attr: proc_macro::TokenStream, item: proc_macro::Token
         #input
     };
     output.into()
-}
\ No newline at end of file
+}
+
+/* -------------------------------------------------------------------------- */
+/*                              #[derive(Topic)]                              */
+/* -------------------------------------------------------------------------- */
+
+use darling::{FromDeriveInput};
+
+#[derive(Debug, Default, FromDeriveInput)]
+#[darling(attributes(topic))]
+struct TopicAttr {
+    #[darling(default)]
+    rename: Option<String>,
+    #[darling(default)]
+    item: Option<syn::Ident>,
+}
+
+/// Implements `toy_rpc::pubsub::Topic` trait for a type. The type will also be
+/// the  associated type `Topic::Item`, and thus the derived type must implement
+/// both `serde::Serialize` and `serde::Deserialize`.
+#[proc_macro_derive(Topic, attributes(topic))]
+pub fn derive_topic(item: proc_macro::TokenStream) -> proc_macro::TokenStream {
+
+    let input = syn::parse_macro_input!(item as syn::DeriveInput);
+    let ident = input.ident.clone();
+    let (topic, item) = {
+        let result = TopicAttr::from_derive_input(&input);
+        println!("{:?}", &result);
+        if let Ok(topic_attr) = result {
+            let t = match topic_attr.rename {
+                Some(s) => s,
+                None => ident.to_string()
+            };
+            let i = match topic_attr.item {
+                Some(i) => i,
+                None => ident.clone()
+            };
+            (t, i)
+        } else {
+            (ident.to_string(), ident.clone())
+        }
+    };
+
+    let output = quote::quote! {
+        impl toy_rpc::pubsub::Topic for #ident {
+            type Item = #item;
+
+            fn topic() -> String {
+                String::from(#topic)
+            }
+        }
+    };
+    output.into()
+}
